﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using StrategyInterface;
using System.Drawing;

namespace SimpleMA
{
    /// <summary>
    /// Торговая стратегия SimpleMA
    /// </summary>
    public class SimpleMA : StrategyBase, IStrategy
    {
        #region Конструктор

        /// <summary>
        /// Инициализирует новый экземпляр класса <see cref="SimpleMA"/>
        /// </summary>
        public SimpleMA()
        {
            // Создаем группу элементов с идентификаторами графиков
            Controls.Add("Group1", new S_GroupBox("Идентификаторы графиков"));
            // Поля для задания идентификаторов графика баров и индикаторов
            Controls.Add("Price", new S_GraphID_TextBox() { LabelText = "Идентификатор графика цены:", Text = "PRICE", Width = 152, XOffset = -40, Itype = IndicatorType.Price });
            Controls.Add("MA",    new S_GraphID_TextBox() { LabelText = "Идентификатор Moving Average:", Text = "MA", Width = 152, XOffset = -40, Itype = IndicatorType.MovingAverage });
            // Начинаем новую группу элементов
            // Сигналы на открытие и закрытие позиции
            Controls.Add("Group2", new S_GroupBox("Сигналы"));
            // Сигналы на открытие и закрытие позиции
            Controls.Add("positionOpen", new S_ComboBox() { LabelText = "Сигнал открытия позиции:", Width = 122, XOffset = -10, Items = new string[] { "Нет", "По пересечению", "По направлению", "С подтверждением", "По вылету" }, SelectedItem = 0, AllowEdit = false });
            Controls.Add("positionClose",new S_ComboBox() { LabelText = "Сигнал закрытия позиции:", Width = 122, XOffset = -10, Items = new string[] { "Нет", "По пересечению", "По направлению", "С подтверждением", "По вылету" }, SelectedItem = 0, AllowEdit = false });
            // Дополнительные условия механической торговой системы
            Controls.Add("Group3", new S_GroupBox("Дополнительные параметры"));
            Controls.Add("pipsOpen",     new S_TextBox()  { LabelText = "Вылет для открытия позиции, пункты:", Text = "0", Width = 60, XOffset = 52, ValueType = TextBoxValueType.Numeric });
            Controls.Add("pipsClose",    new S_TextBox()  { LabelText = "Вылет для закрытия позиции, пункты:", Text = "0", Width = 60, XOffset = 52, ValueType = TextBoxValueType.Numeric });
            Controls.Add("exitExtreme",  new S_CheckBox("Выход по экстремумам", false));
            Controls.Add("currentTrade", new S_CheckBox("Торговля по значениям открытой свечи", false));
            // Последняя группа элементов
            Controls.Add("Group4", new S_GroupBox("Информационные поля"));
            // Поля для вывода информации о используемых в работе значений
            Controls.Add("maValue",      new S_InfoBox() { LabelText = "Значение Moving Average:", Width = 122, XOffset = -10 });
        }

        #endregion

        #region Функции обратного вызова

        /// <summary>
        /// Вызывается при изменении цены последней сделки
        /// </summary>
        /// <param name="price">Цена последней сделки</param>
        public override void OnPrice(decimal price)
        {
            Main();
        }

        /// <summary>
        /// Выполняется с заданным интервалом (по умолчанию 1 сек)
        /// </summary>
        public override void OnTimer()
        {
            Main();
        }

        #endregion

        #region Main()

        // Флаги подсветки полей ввода идентификаторов индикаторов
        bool ma_is_error    = false;
        bool price_is_error = false;

        private void Main()
        {   
            // Очищаем ошибки предыдущей итерации
            Log.LastError = string.Empty;

            // Получаем информацию о бумаге
            ISecurityInfo SecInfo = Data.getSecurityInfo();
            if (SecInfo == null)
            {
                Log.Error(string.Format("Не удалось получить информацию по бумаге {0}/{1}, проверьте параметры инструмента",
                                Settings.SecCode, Settings.ClassCode));
                return;
            }

            // Получаем цены
            IPriceInfo Prices = Data.getPrices();
            if (Prices == null || Prices.LastPrice == 0)
            {
                Log.Error(string.Format("Не удалось получить информацию по ценам для {0}/{1}, проверьте параметры инструмента",
                                Settings.SecCode, Settings.ClassCode));
                return;
            }

            // Получаем свечи ценового графика
            string PriceIndID = Data.getControlValue<string>("Price");
            string get_candlese_error = string.Empty;
            ICandleInfo[] Candles = Data.getPriceCandles(PriceIndID, 3, out get_candlese_error);

            if (get_candlese_error != string.Empty)
            {
                if(!price_is_error)
                {
                    price_is_error = true;
                    Data.setControlColor("Price", Color.Yellow, Color.Red);
                }
                Log.Error(get_candlese_error); //не удалось получить свечи
                return;
            }
            else if (price_is_error)
            {
                Data.setControlColor("Price", SystemColors.Window, SystemColors.WindowText);
            }

            // Идентификатор MA
            string MA_name = Data.getControlValue<string>("MA");

            string get_MA_error = string.Empty;
            IIndicatorValueEx[] MA_data = Data.getIndicatorValuesEx(MA_name, 3, out get_MA_error);
            if (get_MA_error != string.Empty)
            {
                if (!ma_is_error)
                {
                    ma_is_error = true;
                    Data.setControlColor("MA", Color.Yellow, Color.Red);
                }
                Log.Error(get_MA_error);
                return;
            }
            else if(ma_is_error)
            {
                Data.setControlColor("MA", SystemColors.Window, SystemColors.WindowText);
            }

            // Проверка входных данных
            if (!CheckData(Candles, Prices, MA_data)) return;

            // Очищаем информацию об ошибках
            Log.ResetError();

            // Выводим данные в информационные поля о индикаторах --------------
            Data.setInfoBoxValue("maValue", MA_data[0][0].ToString("F6"));

            // Получаем сигнал стратегии и передаем его в модуль торговли
            Trade.SetSignal(GetSignal(Candles, new decimal[] { MA_data[0][0], MA_data[1][0], MA_data[2][0] }));
        }

        #endregion

        #region GetSignal

        private StrategySignal GetSignal(ICandleInfo[] Candles, decimal[] MA)
        {
            // Получаем значения контролов
            int     positionOpen  = Data.getControlValue<int>("positionOpen");
            int     positionClose = Data.getControlValue<int>("positionClose");
            decimal pipsOpen      = Helper.toDecimal(Data.getControlValue<string>("pipsOpen"));
            decimal pipsClose     = Helper.toDecimal(Data.getControlValue<string>("pipsClose"));
            bool    exitExtreme   = Data.getControlValue<bool>("exitExtreme");
            // Признак торговли по значениям открытой (последней) свечи
            int     barIndex      = (Data.getControlValue<bool>("currentTrade")) ? 0 : 1;

            OpenSignal Open = OpenSignal.None;
            
            bool LongClose  = false;
            bool ShortClose = false;

            // -----------------------------
            // Условия ОТКРЫТИЯ позиций
            // -----------------------------
            switch (positionOpen)
            {
                case 0: // Нет
                    break;
                case 1: // Открытие по пересечению
                    if (Candles[barIndex].Close > MA[barIndex])
                        Open = OpenSignal.Long;
                    else if (Candles[barIndex].Close < MA[barIndex])
                        Open = OpenSignal.Short;
                    break;
                case 2: // Открытие по направлению
                    if (MA[barIndex] > MA[barIndex + 1])
                        Open = OpenSignal.Long;
                    else if (MA[barIndex] < MA[barIndex + 1])
                        Open = OpenSignal.Short;
                    break;
                case 3: // Открытие с подтверждением
                    if (Candles[barIndex].Close > MA[barIndex] && Candles[barIndex].Open > MA[barIndex])
                        Open = OpenSignal.Long;
                    else if (Candles[barIndex].Close < MA[barIndex] && Candles[barIndex].Open < MA[barIndex])
                        Open = OpenSignal.Short;
                    break;
                case 4: // Открытие по вылету
                    if (Candles[barIndex].Close >= MA[barIndex] + pipsOpen)
                        Open = OpenSignal.Long;
                    else if (Candles[barIndex].Close <= MA[barIndex] - pipsOpen)
                        Open = OpenSignal.Short;
                    break;
            }
            // -----------------------------
            // Условия ЗАКРЫТИЯ позиций
            // -----------------------------
            switch (positionClose)
            {
                case 0: // Нет
                    break;
                case 1: // Закрытие по пересечению
                    if (Candles[barIndex].Close < MA[barIndex])
                        LongClose = true;
                    if (Candles[barIndex].Close > MA[barIndex])
                        ShortClose = true;
                    break;
                case 2: // Закрытие по направлению
                    if (MA[barIndex] < MA[barIndex + 1])
                        LongClose = true;
                    if (MA[barIndex] > MA[barIndex + 1])
                        ShortClose = true;
                    break;
                case 3: // Закрытие с подтверждением
                    if (Candles[barIndex].Close < MA[barIndex] && Candles[barIndex].Open < MA[barIndex])
                        LongClose = true;
                    if (Candles[barIndex].Close > MA[barIndex] && Candles[barIndex].Open > MA[barIndex])
                        ShortClose = true;
                    break;
                case 4: // Закрытие по вылету
                    if (Candles[barIndex].Close <= MA[barIndex] - pipsClose)
                        LongClose = true;
                    if (Candles[barIndex].Close >= MA[barIndex] + pipsClose)
                        ShortClose = true;
                    break;
            }
            // -----------------------------

            // Условия опции "Выход по экстремумам" ----------------------------------------
            if (exitExtreme)
            {
                if (Settings.TradesOnBarClose)
                {
                    barIndex = 0;
                }
                if (Candles[barIndex].High < Candles[barIndex + 1].High)
                    LongClose = true;
                if (Candles[barIndex].Low > Candles[barIndex + 1].Low)
                    ShortClose = true; 
            }

            return new StrategySignal(Open, LongClose, ShortClose);
        }

        #endregion

        #region Проверка данных

        // Проверка данных
        private bool CheckData(ICandleInfo[] Candles, IPriceInfo Prices, IIndicatorValueEx[] MA)
        {
            // Проверяем значение цены последней сделки
            if (Prices.LastPrice <= 0)
            {
                Log.Error(string.Format("Неверное значение цены последней сделки {0}. Проверьте параметры инструмента.", 
                    Prices.LastPrice));
                return false;
            }

            // Проверяем значения цен баров
            if (Candles.Length < 3)
            {
                Log.Error(string.Format("Недостаточное количество свечей. Проверьте настройки графика цены. (свечи: {0})", 
                    Candles.Length));
                return false;
            }

            if (Candles[0].Open <= 0 || Candles[1].Close <= 0 || Candles[2].Close <= 0)
            {
                Log.Error(string.Format("Неверные значения цен свечей. Проверьте настройки графика цены. ({0} {1} {2})", 
                    Candles[0].Open, Candles[1].Close, Candles[2].Close));
                return false;
            }

            int TimeFrame = Helper.GetTimeFrame(Candles);

            // Проверяем значение временного интервала баров
            if (TimeFrame == 0)
            {
                Log.Error("Таймфрейм равен 0. Проверьте настройки графика свечей.");
                return false;
            }

            // Проверяем значения индикаторов
            if (MA.Length < 3 || MA.Any(a => a.Length != 1))
            {
                Log.Error(string.Format("Недостаточное количество значений Moving Average. Проверьте настройки индикатора. " +
                    "(свечи: {0}, линии: {1})", MA.Length, MA.Length > 0 ? MA[0].Length : 0));
                return false;
            }

            if (MA.Any(a => a.Values[0].Value == 0))
            {
                Log.Error(string.Format("Неверные значения Moving Average. Проверьте настройки индикатора. ({0} {1} {2})",
                    MA[0][0], MA[1][0], MA[2][0]));
                return false;
            }
            return true;
        }

        #endregion
    }
}
